@@ -9,9 +9,11 @@
  */
 
 #include "include/JobQueue.hh"
+#include "include/modules/pythonmonkey/pythonmonkey.hh"
 
 #include "include/PyEventLoop.hh"
 #include "include/pyTypeFactory.hh"
+#include "include/PromiseType.hh"
 
 #include <Python.h>
 
@@ -24,8 +26,9 @@ JobQueue::JobQueue(JSContext *cx) {
   finalizationRegistryCallbacks = new JS::PersistentRooted<FunctionVector>(cx);   // Leaks but it's OK since freed at process exit
 }
 
-JSObject *JobQueue::getIncumbentGlobal(JSContext *cx) {
-  return JS::CurrentGlobalOrNull(cx);
+bool JobQueue::getHostDefinedData(JSContext *cx, JS::MutableHandle<JSObject *> data) const {
+  data.set(nullptr); // We don't need the host defined data
+  return true; // `true` indicates no error
 }
 
 bool JobQueue::enqueuePromiseJob(JSContext *cx,
@@ -57,7 +60,12 @@ void JobQueue::runJobs(JSContext *cx) {
 
 bool JobQueue::empty() const {
   // TODO (Tom Tang): implement using `get_running_loop` and getting job count on loop???
-  throw std::logic_error("JobQueue::empty is not implemented\n");
+  return true; // see https://hg.mozilla.org/releases/mozilla-esr128/file/tip/js/src/builtin/Promise.cpp#l6946
+}
+
+bool JobQueue::isDrainingStopped() const {
+  // TODO (Tom Tang): implement this by detecting if the Python event-loop is still running
+  return false;
 }
 
 js::UniquePtr<JS::JobQueue::SavedJobQueue> JobQueue::saveJobQueue(JSContext *cx) {
@@ -72,6 +80,7 @@ js::UniquePtr<JS::JobQueue::SavedJobQueue> JobQueue::saveJobQueue(JSContext *cx)
 bool JobQueue::init(JSContext *cx) {
   JS::SetJobQueue(cx, this);
   JS::InitDispatchToEventLoop(cx, dispatchToEventLoop, cx);
+  JS::SetPromiseRejectionTrackerCallback(cx, promiseRejectionTracker);
   return true;
 }
 
@@ -113,10 +122,48 @@ bool sendJobToMainLoop(PyObject *pyFunc) {
   }
   loop.enqueue(pyFunc);
 
+  loop._loop = nullptr; // the `Py_XDECREF` Python API call in `PyEventLoop`'s destructor will not be accessible once we hand over the GIL by `PyGILState_Release`
   PyGILState_Release(gstate);
   return true;
 }
 
+void JobQueue::promiseRejectionTracker(JSContext *cx,
+  bool mutedErrors,
+  JS::HandleObject promise,
+  JS::PromiseRejectionHandlingState state,
+  [[maybe_unused]] void *privateData) {
+
+  // We only care about unhandled Promises
+  if (state != JS::PromiseRejectionHandlingState::Unhandled) {
+    return;
+  }
+  // If the `mutedErrors` option is set to True in `pm.eval`, eval errors or unhandled rejections should be ignored.
+  if (mutedErrors) {
+    return;
+  }
+
+  // Test if there's no user-defined (or pmjs defined) exception handler on the Python event-loop
+  PyEventLoop loop = PyEventLoop::getRunningLoop();
+  if (!loop.initialized()) return;
+  PyObject *customHandler = PyObject_GetAttrString(loop._loop, "_exception_handler"); // see https://github.com/python/cpython/blob/v3.9.16/Lib/asyncio/base_events.py#L1782
+  if (customHandler == Py_None) { // we only have the default exception handler
+    // Set an exception handler to the event-loop
+    PyObject *pmModule = PyImport_ImportModule("pythonmonkey");
+    PyObject *exceptionHandler = PyObject_GetAttrString(pmModule, "simpleUncaughtExceptionHandler");
+    PyObject_CallMethod(loop._loop, "set_exception_handler", "O", exceptionHandler);
+    Py_DECREF(pmModule);
+    Py_DECREF(exceptionHandler);
+  }
+  Py_DECREF(customHandler);
+
+  // Go ahead and send this unhandled Promise rejection to the exception handler on the Python event-loop
+  PyObject *pyFuture = PromiseType::getPyObject(cx, promise); // ref count == 2
+  // Unhandled Future object calls the event-loop exception handler in its destructor (the `__del__` magic method)
+  // See https://github.com/python/cpython/blob/v3.9.16/Lib/asyncio/futures.py#L108
+  //  or https://github.com/python/cpython/blob/v3.9.16/Modules/_asynciomodule.c#L1457-L1467 (It will actually use the C module by default, see futures.py#L417-L423)
+  Py_DECREF(pyFuture); // decreasing the reference count from 2 to 1, leaving one for the `onResolved` callback in `PromiseType::getPyObject`, which will be called very soon and clean up the reference
+}
+
 void JobQueue::queueFinalizationRegistryCallback(JSFunction *callback) {
   mozilla::Unused << finalizationRegistryCallbacks->append(callback);
 }
